#include "DippyPythonUtils.h"
#include "UtilMacros.h"
#include "UtilMacrosDecomp.h"

#include <assert.h>

//===========================================================================//

// Some convenience functions for converting between Python objects and 
// C/C++ data structures

/**
 * Convert a double array to a Python tuple list
 *
 * The list is (Python object, value) tuples
 *
 * Returns Python tuple list with length = pList length
 */
PyObject * pyTupleList_FromDoubleArray(const double * values, PyObject *pList) {
	int len = PyObject_Length(pList);
	PyObject *pTupleList = PyList_New(len), *pPair, *pObj;

	for (int i=0; i<len; i++) {
		pPair = PyTuple_New(2);
		pObj = PyList_GetItem(pList, i);
		Py_XINCREF(pObj);
		PyTuple_SetItem(pPair, 0, pObj);
		PyTuple_SetItem(pPair, 1, PyFloat_FromDouble(values[i]));
        PyList_SetItem(pTupleList, i, pPair);
    }

    return pTupleList;
}

/**
 * Convert a column dictionary to a (int, double) vector
 *
 * The dictionary has columns as keys
 * and coefficients as values
 *
 */
void pyColDict_AsPairedVector(PyObject *pColDict, vector<pair<int, double>> & vec, map<PyObject*, int> indices) {
	int len = PyObject_Length(pColDict);

	vec.clear();

	PyObject *pKeys = PyDict_Keys(pColDict), *pCol;
	double value;
	int index;
	for (int i=0; i<len; i++) {
		pCol = PyList_GetItem(pKeys, i);
		value = PyFloat_AsDouble(PyDict_GetItem(pColDict, pCol));
		index = indices[pCol];
		if ( (index < 0) || (index >= indices.size()) ) {
			string name = PyString_AsString(PyObject_CallMethod(pCol, "__str__", NULL));
			throw UtilException("Bad index for " + name, "pyTupleList_AsPairedVector", "DippyPythonUtils");
		}
		vec.push_back(pair<int, double>(index, value));
	}
}

/**
 * Convert a column dictionary to packed arrays
 *
 * The dictionary has columns as keys
 * and coefficients as values
 *
 * Returns length of index and value arrays
 */
int pyColDict_AsPackedArrays(PyObject *pColDict, map<PyObject*, int> indices, int ** inds, double ** vals) {
	int len = PyObject_Length(pColDict);

	*inds = new int[len];
	*vals = new double[len];
	PyObject *pKeys = PyDict_Keys(pColDict);
	PyObject *pCol;
	double value;
	int index;
	for (int i=0; i<len; i++) {
		pCol = PyList_GetItem(pKeys, i);
		value = PyFloat_AsDouble(PyDict_GetItem(pColDict, pCol));
		index = indices[pCol];
		if ( (index < 0) || (index >= indices.size()) ) {
			string name = PyString_AsString(PyObject_CallMethod(pCol, "getName", NULL));
			throw UtilException("Bad index for " + name, "pyTupleList_AsPackedArray", "DippyPythonUtils");
		}
		(*inds)[i] = index;
		(*vals)[i] = value;
	}

	return len;
}

/**
 * Convert a list of Python constraints to a CoinPackedMatrix
 *
 * The format of each constraint is as a dictionary with variables as keys
 * and coefficients as values
 */

CoinPackedMatrix* pyConstraints_AsPackedMatrix(PyObject *pRowList,
	map<PyObject*, int> rowIndices, map<PyObject*, int> colIndices) {

	int len = PyObject_Length(pRowList);
	int rowInd, colInd, num;
	string rowName, colName;
	double val, lb, ub;
	PyObject *pRow, *pKeys, *pCol;

	// First get the total number of non-zeros from all the tuples
	int numNZs = 0;
	for (int i=0; i<len; i++) {
		pRow = PyList_GetItem(pRowList, i);
		num = PyObject_Length(pRow);
		numNZs += num;
	}

	// Now read and process the tuples
	int start = 0;
	int * rowInds = new int[numNZs];
	UtilFillN(rowInds, numNZs, -1);
	int * colInds = new int[numNZs];
	UtilFillN(colInds, numNZs, -1);
	double * values = new double[numNZs];
	UtilFillN(values, numNZs, 0.0);

	for (int i=0; i<len; i++) {
		pRow = PyList_GetItem(pRowList, i);
		rowInd = rowIndices[pRow];
		pKeys = PyDict_Keys(pRow);
		num = PyObject_Length(pKeys);
		for (int n=0; n<num; n++) {
			pCol = PyList_GetItem(pKeys, n);
			colInd = colIndices[pCol];
			val = PyFloat_AsDouble(PyDict_GetItem(pRow, pCol));
			rowInds[start + n] = rowInd;
			colInds[start + n] = colInd;
			values[start + n] = val;
		}
		start += num;
	}

	return new CoinPackedMatrix(false, rowInds, colInds, values, numNZs);
}
